mod cli;
mod coords;

mod current_weather;
mod current_weather_printer;
mod ifconfig;
mod ip_api;
mod open_meteo;

use billboard::{Alignment, Billboard};
use std::process::exit;
use structopt::StructOpt;

use cli::Arguments;
use coords::Coordinates;

use current_weather_printer::CurrentWeatherPrinter;
use ifconfig::extract_public_ip;
use ip_api::extract_coords_and_city;
use open_meteo::request_current_weather;

fn main() {
    let opts = Arguments::from_args();
    match opts {
        Arguments::CurrentWeather {
            latitude,
            longitude,
            all,
            is_day,
            temperature,
            windspeed,
            winddirection,
            include_coords,
            include_city,
            clean,
        } => {
            if !all
                && !is_day
                && !temperature
                && !windspeed
                && !winddirection
                && !include_coords
                && !include_city
            {
                eprintln!("[ERROR] Provide at least one parameter to print, check --help");
                exit(1);
            }

            let coordinates: Coordinates;
            let city: Option<String>;

            if latitude.is_some() && longitude.is_some() {
                let latitude = latitude.unwrap();
                let longitude = longitude.unwrap();
                coordinates = Coordinates::new(latitude, longitude);

                if !coordinates.is_correct_latitude() {
                    eprintln!("[ERROR] {} is not a valid latitude", coordinates.latitude);
                    exit(1);
                } else if !coordinates.is_correct_longitude() {
                    eprintln!("[ERROR] {} is not a valid longitude", coordinates.longitude);
                    exit(1);
                }

                city = None;
            } else {
                let ip_result = extract_public_ip();
                let ip = ip_result.unwrap_or_else(|_| {
                    eprintln!("[ERROR] Could not extract public IP from ifconfig");
                    exit(1);
                });

                let coords_result = extract_coords_and_city(&ip);
                let coords = coords_result.unwrap_or_else(|_| {
                    eprintln!("[ERROR] Could not extract IP coordinates from ip_api");
                    exit(1);
                });

                let latitude = coords["lat"].as_f64().unwrap();
                let longitude = coords["lon"].as_f64().unwrap();
                coordinates = Coordinates::new(latitude, longitude);
                city = Some(coords["city"].as_str().unwrap().to_string());
            }

            let result = request_current_weather(&coordinates);
            let current_weather = result.unwrap_or_else(|_| {
                eprintln!(
                    "[ERROR] Requesting CurrentWeather for Lat: {}, Lon: {}",
                    coordinates.latitude, coordinates.longitude
                );
                exit(1);
            });

            let mut string_vec: Vec<String> = Vec::new();

            if !clean {
                let mut title_header: Vec<String> = Vec::new();
                title_header.push(String::from("=== Current weather"));

                if (all || include_city) && city.is_some() {
                    title_header.push(String::from("for"));
                    title_header.push(city.unwrap());
                }

                title_header.push(String::from("==="));
                string_vec.push(title_header.join(" "));
            } else {
                if all || include_coords {
                    string_vec.push(coordinates.latitude.to_string());
                    string_vec.push(coordinates.longitude.to_string());
                }
                if (all || include_city) && city.is_some() {
                    string_vec.push(city.unwrap());
                }
            }

            let current_weather_printer = CurrentWeatherPrinter::new(current_weather, clean);
            if is_day || all {
                string_vec.push(current_weather_printer.extract_day());
            }
            if temperature || all {
                string_vec.push(current_weather_printer.extract_temperature());
            }
            if windspeed || all {
                string_vec.push(current_weather_printer.extract_wind_speed());
            }
            if winddirection || all {
                string_vec.push(current_weather_printer.extract_wind_direction());
            }

            if !clean && (all || include_coords) {
                string_vec.push(format!(
                    "Latitude: {}, Longitude: {}",
                    coordinates.latitude, coordinates.longitude
                ));
            }

            if clean {
                let final_string = string_vec.join(",");
                println!("{final_string}");
            } else {
                string_vec.push(String::from("=== Weather data by Open-Meteo.com ==="));
                let final_string = string_vec.join("\n");
                Billboard::builder()
                    .text_alignment(Alignment::Left)
                    .box_alignment(Alignment::Left)
                    .build()
                    .eprint(final_string);
            }
        }
    }
}
